-- ******************************************
-- Loosely based on RPU project
-- See: https://github.com/Domipheus/RPU.git
-- ******************************************
library IEEE;
use IEEE.STD_LOGIC_1164.all;

library work;
package constants is


--XLEN: Length of integer registers
constant XLEN: integer := 32;
constant XLENM1: integer := XLEN - 1;

--FLEN: Length of float integers
constant FLEN : integer := 32;
constant FLENM1 : integer := FLEN - 1;

-- Address width; used for defining an address vector
CONSTANT ADDR_WIDTH : integer := 5;
CONSTANT ADDR_WIDTHM1 : integer := ADDR_WIDTH - 1;

constant ADDR_RESET:    std_logic_vector(XLENM1 downto 0) :=  X"00000000";
constant ADDR_INTVEC:   std_logic_vector(XLENM1 downto 0) :=  X"00000100";

--Bitmasks
CONSTANT WORD_MASK	 : STD_LOGIC_VECTOR(XLENM1 DOWNTO 0) := x"ffffffff";
CONSTANT HALF_MASK	 : STD_LOGIC_VECTOR(XLENM1 DOWNTO 0) := x"0000ffff";
CONSTANT HALF_UPPER_MASK : STD_LOGIC_VECTOR(XLENM1 DOWNTO 0) := x"ffff0000";
CONSTANT BYTE_MASK	 : STD_LOGIC_VECTOR(XLENM1 DOWNTO 0) := x"000000ff";
CONSTANT BYTE_UPPER_MASK : STD_LOGIC_VECTOR(XLENM1 DOWNTO 0) := x"ff000000";

-- PC unit opcodes
constant PCU_OP_NOP: std_logic_vector(1 downto 0):= "00";
constant PCU_OP_INC: std_logic_vector(1 downto 0):= "01";
constant PCU_OP_ASSIGN: std_logic_vector(1 downto 0):= "10";
constant PCU_OP_RESET: std_logic_vector(1 downto 0):= "11";

-- Instruction Form Offsets
constant OPCODE_START: integer := 6;
constant OPCODE_END: integer := 0;
constant OPCODE_END_2: integer := 2;
constant OPCODE_LEN : integer := OPCODE_START - OPCODE_END_2;

constant RD_START: integer := 11;
constant RD_END: integer := 7;
constant RD_LEN : integer := RD_START - RD_END;

constant FUNCT3_START: integer := 14;
constant FUNCT3_END: integer := 12;
constant FUNCT3_LEN : integer := FUNCT3_START - FUNCT3_END;

constant R1_START: integer := 19;
constant R1_END: integer := 15;
constant R1_LEN : integer := R1_START - R1_END;

constant R2_START: integer := 24;
constant R2_END: integer := 20;
constant R2_LEN : integer := R2_START - R2_END;

constant FUNCT7_START: integer := 31;
constant FUNCT7_END: integer := 25;
constant FUNCT7_LEN : integer := FUNCT7_START - FUNCT7_END;

constant FUNCT2_START : integer := 26;
constant FUNCT2_END : integer := 25;
constant FUNCT2_LEN : integer := FUNCT2_START - FUNCT2_END;

constant R3_START : integer := 31;
constant R3_END : integer := 27;
constant R3_LEN : integer := R3_START - R3_END;

constant IMM_I_START: integer := 31;
constant IMM_I_END: integer := 20;
constant IMM_I_LEN : integer := IMM_I_START - IMM_I_END;

constant IMM_U_START: integer := 31;
constant IMM_U_END: integer := 12;
constant IMM_U_LEN : integer := IMM_U_START - IMM_U_END;

constant IMM_J_LEN : integer := IMM_U_LEN;

constant IMM_S_A_START: integer := 31;
constant IMM_S_A_END: integer := 25;
constant IMM_S_A_LEN : integer := IMM_S_A_START - IMM_S_A_END;

constant IMM_S_B_START: integer := 11;
constant IMM_S_B_END: integer := 7;
constant IMM_S_B_LEN : integer := IMM_S_B_START - IMM_S_B_END;

constant IMM_S_LEN : integer := IMM_S_A_LEN + IMM_S_A_LEN + 1;

constant IMM_B_LEN : integer := IMM_S_LEN;

constant FUNCT4_START : integer := 31;
constant FUNCT4_END : integer := 28;
constant FUNCT4_LEN : integer := FUNCT4_START - FUNCT4_END;

constant FENCE_PRED_START : integer := 27;
constant FENCE_PRED_END : integer := 23;
constant FENCE_PRED_LEN : integer := FENCE_PRED_START - FENCE_PRED_END;

constant FENCE_SUCC_START : integer := 22;
constant FENCE_SUCC_END : integer := 19;
constant FENCE_SUCC_LEN : integer := FENCE_SUCC_START - FENCE_SUCC_END;
-- Opcodes
constant OPCODE_LOAD	: std_logic_vector(4 downto 0) := "00000";
constant OPCODE_FLOAD	: std_logic_vector(4 downto 0) := "00001";
constant OPCODE_MISCMEM	: std_logic_vector(4 downto 0) := "00011";
constant OPCODE_OPIMM	: std_logic_vector(4 downto 0) := "00100";
constant OPCODE_AUIPC	: std_logic_vector(4 downto 0) := "00101";
constant OPCODE_STORE	: std_logic_vector(4 downto 0) := "01000";
constant OPCODE_FSTORE	: std_logic_vector(4 downto 0) := "01001";
constant OPCODE_AMO	: std_logic_vector(4 downto 0) := "01011";
constant OPCODE_OP	: std_logic_vector(4 downto 0) := "01100";
constant OPCODE_LUI	: std_logic_vector(4 downto 0) := "01101";
constant OPCODE_MADD	: std_logic_vector(4 downto 0) := "10000";
constant OPCODE_MSUB	: std_logic_vector(4 downto 0) := "10001";
constant OPCODE_NMADD	: std_logic_vector(4 downto 0) := "10010";
constant OPCODE_NMSUB	: std_logic_vector(4 downto 0) := "10011";
constant OPCODE_FLOAT	: std_logic_vector(4 downto 0) := "10100";
constant OPCODE_VECTOR	: std_logic_vector(4 downto 0) := "10101";
constant OPCODE_BRANCH	: std_logic_vector(4 downto 0) := "11000";
constant OPCODE_JALR	: std_logic_vector(4 downto 0) := "11001";
constant OPCODE_JAL	: std_logic_vector(4 downto 0) := "11011";
constant OPCODE_SYSTEM	: std_logic_vector(4 downto 0) := "11100";

-- Flags
constant F3_BRANCH_BEQ: std_logic_vector(2 downto 0) := "000";
constant F3_BRANCH_BNE: std_logic_vector(2 downto 0) := "001";
constant F3_BRANCH_BLT: std_logic_vector(2 downto 0) := "100";
constant F3_BRANCH_BGE: std_logic_vector(2 downto 0) := "101";
constant F3_BRANCH_BLTU: std_logic_vector(2 downto 0) := "110";
constant F3_BRANCH_BGEU: std_logic_vector(2 downto 0) := "111";

constant F3_JALR: std_logic_vector(2 downto 0) := "000";

constant F3_LOAD_LB: std_logic_vector(2 downto 0) := "000";
constant F3_LOAD_LH: std_logic_vector(2 downto 0) := "001";
constant F3_LOAD_LW: std_logic_vector(2 downto 0) := "010";
constant F3_LOAD_LBU: std_logic_vector(2 downto 0) := "100";
constant F3_LOAD_LHU: std_logic_vector(2 downto 0) := "101";

constant F3_LOAD_FLW: std_logic_vector(2 downto 0) := "010";

constant F2_MEM_LS_SIZE_B: std_logic_vector(1 downto 0) := "00";
constant F2_MEM_LS_SIZE_H: std_logic_vector(1 downto 0) := "01";
constant F2_MEM_LS_SIZE_W: std_logic_vector(1 downto 0) := "10";

constant F3_STORE_SB: std_logic_vector(2 downto 0) := "000";
constant F3_STORE_SH: std_logic_vector(2 downto 0) := "001";
constant F3_STORE_SW: std_logic_vector(2 downto 0) := "010";

constant F3_AMO : std_logic_vector(2 downto 0) := "010";

constant F3_OPIMM_ADDI: std_logic_vector(2 downto 0) := "000";
constant F3_OPIMM_SLTI: std_logic_vector(2 downto 0) := "010";
constant F3_OPIMM_SLTIU: std_logic_vector(2 downto 0) := "011";
constant F3_OPIMM_XORI: std_logic_vector(2 downto 0) := "100";
constant F3_OPIMM_ORI: std_logic_vector(2 downto 0) := "110";
constant F3_OPIMM_ANDI: std_logic_vector(2 downto 0) := "111";

constant F3_OPIMM_SLLI: std_logic_vector(2 downto 0) := "001";
constant F7_OPIMM_SLLI: std_logic_vector(6 downto 0) := "0000000";
constant F3_OPIMM_SRLI: std_logic_vector(2 downto 0) := "101";
constant F7_OPIMM_SRLI: std_logic_vector(6 downto 0) := "0000000";
constant F3_OPIMM_SRAI: std_logic_vector(2 downto 0) := "101";
constant F7_OPIMM_SRAI: std_logic_vector(6 downto 0) := "0100000";

constant OP_ADD	   : std_logic_vector(4 downto 0) := "00000";
constant OP_SUB    : std_logic_vector(4 downto 0) := "00001";
constant OP_SLL    : std_logic_vector(4 downto 0) := "00010";
constant OP_SLT    : std_logic_vector(4 downto 0) := "00011";
constant OP_SLTU   : std_logic_vector(4 downto 0) := "00100";
constant OP_XOR    : std_logic_vector(4 downto 0) := "00101";
constant OP_SRL    : std_logic_vector(4 downto 0) := "00110";
constant OP_SRA    : std_logic_vector(4 downto 0) := "00111";
constant OP_OR     : std_logic_vector(4 downto 0) := "01000";
constant OP_AND    : std_logic_vector(4 downto 0) := "01001";
constant OP_MUL	   : std_logic_vector(4 downto 0) := "01010";
constant OP_MULH   : std_logic_vector(4 downto 0) := "01011";
constant OP_MULHSU : std_logic_vector(4 downto 0) := "01100";
constant OP_MULHU  : std_logic_vector(4 downto 0) := "01101";
constant OP_DIV    : std_logic_vector(4 downto 0) := "01110";
constant OP_DIVU   : std_logic_vector(4 downto 0) := "01111";
constant OP_REM    : std_logic_vector(4 downto 0) := "10000";
constant OP_REMU   : std_logic_vector(4 downto 0) := "10001";
constant OP_LEN    : integer := 4;

constant F3_OP_ADD: std_logic_vector(2 downto 0) := "000";
constant F7_OP_ADD: std_logic_vector(6 downto 0) := "0000000";
constant F3_OP_SUB: std_logic_vector(2 downto 0) := "000";
constant F7_OP_SUB: std_logic_vector(6 downto 0) := "0100000";
constant F3_OP_SLL: std_logic_vector(2 downto 0) := "001";
constant F7_OP_SLL: std_logic_vector(6 downto 0) := "0000000";
constant F3_OP_SLT: std_logic_vector(2 downto 0) := "010";
constant F7_OP_SLT: std_logic_vector(6 downto 0) := "0000000";
constant F3_OP_SLTU: std_logic_vector(2 downto 0) := "011";
constant F7_OP_SLTU: std_logic_vector(6 downto 0) := "0000000";
constant F3_OP_XOR: std_logic_vector(2 downto 0) := "100";
constant F7_OP_XOR: std_logic_vector(6 downto 0) := "0000000";
constant F3_OP_SR: std_logic_vector(2 downto 0) := "101";
constant F7_OP_SRL: std_logic_vector(6 downto 0) := "0000000";
constant F7_OP_SRA: std_logic_vector(6 downto 0) := "0100000";
constant F3_OP_OR: std_logic_vector(2 downto 0) := "110";
constant F7_OP_OR: std_logic_vector(6 downto 0) := "0000000";
constant F3_OP_AND: std_logic_vector(2 downto 0) := "111";
constant F7_OP_AND: std_logic_vector(6 downto 0) := "0000000";

-- RV32M Extension
constant F7_OP_M_EXT: std_logic_vector(6 downto 0) := "0000001";
constant F3_OP_M_MUL: std_logic_vector(2 downto 0) := "000";
constant F3_OP_M_MULH: std_logic_vector(2 downto 0) := "001";
constant F3_OP_M_MULHSU: std_logic_vector(2 downto 0) := "010";
constant F3_OP_M_MULHU: std_logic_vector(2 downto 0) := "011";
constant F3_OP_M_DIV: std_logic_vector(2 downto 0) := "100";
constant F3_OP_M_DIVU: std_logic_vector(2 downto 0) := "101";
constant F3_OP_M_REM: std_logic_vector(2 downto 0) := "110";
constant F3_OP_M_REMU: std_logic_vector(2 downto 0) := "111";

-- bit 0 of the OP definitions denote unsigned ops; same as above
constant ALU_INT32_DIV_OP_UNSIGNED_BIT: integer := 0;
constant ALU_INT32_DIV_OP_DIV: std_logic_vector(1 downto 0) := "00";
constant ALU_INT32_DIV_OP_DIVU: std_logic_vector(1 downto 0) := "01";
constant ALU_INT32_DIV_OP_REM: std_logic_vector(1 downto 0) := "10";
constant ALU_INT32_DIV_OP_REMU: std_logic_vector(1 downto 0) := "11";

constant FLOAT_INST : std_logic_vector(1 downto 0) := "00";
constant HFLOAT_INST : std_logic_vector(1 downto 0) := "10";

constant FLOAT_OP_MADD : std_logic_vector(4 downto 0) := "00000";
constant FLOAT_OP_MSUB : std_logic_vector(4 downto 0) := "00001";
constant FLOAT_OP_NMSUB: std_logic_vector(4 downto 0) := "00010";
constant FLOAT_OP_NMADD: std_logic_vector(4 downto 0) := "00011";
constant FLOAT_OP_ADD  : std_logic_vector(4 downto 0) := "00100";
constant FLOAT_OP_SUB  : std_logic_vector(4 downto 0) := "00101";
constant FLOAT_OP_MUL  : std_logic_vector(4 downto 0) := "00110";
constant FLOAT_OP_DIV  : std_logic_vector(4 downto 0) := "00111";
constant FLOAT_OP_SQRT : std_logic_vector(4 downto 0) := "01000";
constant FLOAT_OP_SGNJ : std_logic_vector(4 downto 0) := "01001";
constant FLOAT_OP_SGNJN: std_logic_vector(4 downto 0) := "01010";
constant FLOAT_OP_SGNJX: std_logic_vector(4 downto 0) := "01011";
constant FLOAT_OP_MIN  : std_logic_vector(4 downto 0) := "01100";
constant FLOAT_OP_MAX  : std_logic_vector(4 downto 0) := "01101";
constant FLOAT_OP_CVTW : std_logic_vector(4 downto 0) := "01110";
constant FLOAT_OP_CVTWU: std_logic_vector(4 downto 0) := "01111";
constant FLOAT_OP_MVX  : std_logic_vector(4 downto 0) := "10000";
constant FLOAT_OP_EQ   : std_logic_vector(4 downto 0) := "10001";
constant FLOAT_OP_LT   : std_logic_vector(4 downto 0) := "10010";
constant FLOAT_OP_LE   : std_logic_vector(4 downto 0) := "10011";
constant FLOAT_OP_CLASS: std_logic_vector(4 downto 0) := "10100";
constant FLOAT_OP_CVTS : std_logic_vector(4 downto 0) := "10101";
constant FLOAT_OP_CVTSU: std_logic_vector(4 downto 0) := "10110";
constant FLOAT_OP_MVW  : std_logic_vector(4 downto 0) := "10111";
constant FLOAT_OP_LEN : integer := 5;
constant FLOAT_OP_LENM1 : integer := FLOAT_OP_LEN - 1;

constant R3_OP_FADD   : std_logic_vector(4 downto 0) := "00000";
constant R3_OP_FSUB   : std_logic_vector(4 downto 0) := "00001";
constant R3_OP_FMUL   : std_logic_vector(4 downto 0) := "00010";
constant R3_OP_FDIV   : std_logic_vector(4 downto 0) := "00011";
constant R3_OP_FSIGN  : std_logic_vector(4 downto 0) := "00100";
constant F3_OP_FSGNJ  : std_logic_vector(2 downto 0) := "000";
constant F3_OP_FSGNJN : std_logic_vector(2 downto 0) := "001";
constant F3_OP_FSGNJX : std_logic_vector(2 downto 0) := "010";
constant R3_OP_FMINMAX: std_logic_vector(4 downto 0) := "00101";
constant F3_OP_FMIN   : std_logic_vector(2 downto 0) := "000";
constant F3_OP_FMAX   : std_logic_vector(2 downto 0) := "001";
constant R3_OP_FSQRT  : std_logic_vector(4 downto 0) := "01011";
constant R2_OP_FSQRT  : std_logic_vector(4 downto 0) := "00000";
constant R3_OP_FCOMP  : std_logic_vector(4 downto 0) := "10100";
constant F3_OP_FEQ    : std_logic_vector(2 downto 0) := "010";
constant F3_OP_FLT    : std_logic_vector(2 downto 0) := "001";
constant F3_OP_FLE    : std_logic_vector(2 downto 0) := "000";
constant R3_OP_FCONVW : std_logic_vector(4 downto 0) := "11000";
constant R2_OP_FCVTW  : std_logic_vector(4 downto 0) := "00000";
constant R2_OP_FCVTWU : std_logic_vector(4 downto 0) := "00001";
constant R3_OP_FCONVS : std_logic_vector(4 downto 0) := "11010";
constant R3_OP_FCLASS : std_logic_vector(4 downto 0) := "11100";
constant F3_OP_FMVX   : std_logic_vector(2 downto 0) := "000";
constant F3_OP_FCLASS : std_logic_vector(2 downto 0) := "001";
constant R3_OP_FMV    : std_logic_vector(4 downto 0) := "11110";

constant F3_MISCMEM_FENCE: std_logic_vector(2 downto 0) := "000";
constant F3_MISCMEM_FENCEI: std_logic_vector(2 downto 0) := "001";
constant F4_FENCE : std_logic_vector(3 downto 0) := "0000";
constant F4_TSO : std_logic_vector(3 downto 0) := "1000";
constant R_DEST_FENCE : std_logic_vector(4 downto 0) := "00000";
constant R1_FENCE : std_logic_vector(4 downto 0) := "00000";

constant F3_SYSTEM_ECALL: std_logic_vector(2 downto 0) := "000";
constant IMM_I_SYSTEM_ECALL: std_logic_vector(11 downto 0) := "000000000000";
constant IMM_U_ECALL : std_logic_vector(24 downto 0) := "0000000000000000000000000";
constant F3_SYSTEM_EBREAK: std_logic_vector(2 downto 0) := "000";
constant IMM_I_SYSTEM_EBREAK: std_logic_vector(11 downto 0) := "000000000001";
constant IMM_U_EBREAK : std_logic_vector(24 downto 0) := "0000000000010000000000000";
constant IMM_U_NTO : std_logic_vector(24 downto 0) := "0000000011010000000000000";
constant IMM_U_STO : std_logic_vector(24 downto 0) := "0000000111010000000000000";
constant F3_SYSTEM_CSRRW: std_logic_vector(2 downto 0) := "001";
constant F3_SYSTEM_CSRRS: std_logic_vector(2 downto 0) := "010";
constant F3_SYSTEM_CSRRC: std_logic_vector(2 downto 0) := "011";
constant F3_SYSTEM_CSRRWI: std_logic_vector(2 downto 0) := "101";
constant F3_SYSTEM_CSRRSI: std_logic_vector(2 downto 0) := "110";
constant F3_SYSTEM_CSRRCI: std_logic_vector(2 downto 0) := "111";


constant F3_PRIVOP: std_logic_vector(2 downto 0) := "000";

constant F7_PRIVOP_URET: std_logic_vector(6 downto 0)       := "0000000";
constant F7_PRIVOP_SRET_WFI: std_logic_vector(6 downto 0)   := "0001000";
constant F7_PRIVOP_MRET: std_logic_vector(6 downto 0)       := "0011000";
constant F7_PRIVOP_SFENCE_VMA: std_logic_vector(6 downto 0) := "0001001";

constant RD_PRIVOP: std_logic_vector(4 downto 0) := "00000";

constant R2_PRIV_RET: std_logic_vector(4 downto 0) := "00010";
constant R2_PRIV_WFI: std_logic_vector(4 downto 0) := "00101";


constant EXCEPTION_INT_USER_SOFTWARE:       std_logic_vector(XLENM1 downto 0):=  X"80000000";
constant EXCEPTION_INT_SUPERVISOR_SOFTWARE: std_logic_vector(XLENM1 downto 0):=  X"80000001";
--constant EXCEPTION_INT_RESERVED:          std_logic_vector(XLENM1 downto 0):=  X"80000002";
constant EXCEPTION_INT_MACHINE_SOFTWARE:    std_logic_vector(XLENM1 downto 0):=  X"80000003";
constant EXCEPTION_INT_USER_TIMER:          std_logic_vector(XLENM1 downto 0):=  X"80000004";
constant EXCEPTION_INT_SUPERVISOR_TIMER:    std_logic_vector(XLENM1 downto 0):=  X"80000005";
--constant EXCEPTION_INT_RESERVED:          std_logic_vector(XLENM1 downto 0):=  X"80000006";
constant EXCEPTION_INT_MACHINE_TIMER:       std_logic_vector(XLENM1 downto 0):=  X"80000007";
constant EXCEPTION_INT_USER_EXTERNAL:       std_logic_vector(XLENM1 downto 0):=  X"80000008";
constant EXCEPTION_INT_SUPERVISOR_EXTERNAL: std_logic_vector(XLENM1 downto 0):=  X"80000009";
--constant EXCEPTION_INT_RESERVED:          std_logic_vector(XLENM1 downto 0):=  X"8000000a";
constant EXCEPTION_INT_MACHINE_EXTERNAL:    std_logic_vector(XLENM1 downto 0):=  X"8000000b";

constant EXCEPTION_INSTRUCTION_ADDR_MISALIGNED: std_logic_vector(XLENM1 downto 0):=  X"00000000";
constant EXCEPTION_INSTRUCTION_ACCESS_FAULT:    std_logic_vector(XLENM1 downto 0):=  X"00000001";
constant EXCEPTION_INSTRUCTION_ILLEGAL:         std_logic_vector(XLENM1 downto 0):=  X"00000002";
constant EXCEPTION_BREAKPOINT:                  std_logic_vector(XLENM1 downto 0):=  X"00000003";
constant EXCEPTION_LOAD_ADDRESS_MISALIGNED:     std_logic_vector(XLENM1 downto 0):=  X"00000004";
constant EXCEPTION_LOAD_ACCESS_FAULT:           std_logic_vector(XLENM1 downto 0):=  X"00000005";
constant EXCEPTION_STORE_AMO_ADDRESS_MISALIGNED:std_logic_vector(XLENM1 downto 0):=  X"00000006";
constant EXCEPTION_STORE_AMO_ACCESS_FAULT:      std_logic_vector(XLENM1 downto 0):=  X"00000007";
constant EXCEPTION_ENVIRONMENT_CALL_FROM_UMODE: std_logic_vector(XLENM1 downto 0):=  X"00000008";
constant EXCEPTION_ENVIRONMENT_CALL_FROM_SMODE: std_logic_vector(XLENM1 downto 0):=  X"00000009";
--constant EXCEPTION_RESERVED:                  std_logic_vector(XLENM1 downto 0):=  X"0000000a";
constant EXCEPTION_ENVIRONMENT_CALL_FROM_MMODE: std_logic_vector(XLENM1 downto 0):=  X"0000000b";
constant EXCEPTION_INSTRUCTION_PAGE_FAULT:      std_logic_vector(XLENM1 downto 0):=  X"0000000c";
constant EXCEPTION_LOAD_PAGE_FAULT:             std_logic_vector(XLENM1 downto 0):=  X"0000000d";
--constant EXCEPTION_RESERVED:                  std_logic_vector(XLENM1 downto 0):=  X"0000000e";
constant EXCEPTION_STORE_AMO_PAGE_FAULT:        std_logic_vector(XLENM1 downto 0):=  X"0000000f";

constant CSR_ADDR_PRIVILEGE_BIT_START:  integer := 9;
constant CSR_ADDR_PRIVILEGE_BIT_END:    integer := 8;
constant CSR_ADDR_PRIVILEGE_USER:       std_logic_vector(1 downto 0):= "00";
constant CSR_ADDR_PRIVILEGE_SUPERVISOR: std_logic_vector(1 downto 0):= "01";
constant CSR_ADDR_PRIVILEGE_RESERVED:   std_logic_vector(1 downto 0):= "10";
constant CSR_ADDR_PRIVILEGE_MACHINE:    std_logic_vector(1 downto 0):= "11";

constant CSR_ADDR_ACCESS_BIT_START: integer := 11;
constant CSR_ADDR_ACCESS_BIT_END:   integer := 10;
constant CSR_ADDR_ACCESS_READONLY:  std_logic_vector(1 downto 0):= "11";


-- CSR Opcodes:
-- 0 - CSR Written
-- 1 - CSR Read
-- 2..3 0 operation
--    01 - read or write whole XLEN
--    10 - Set bits
--    11 - clear bits
-- 4 - immediate or register

constant CSR_OP_BITS_READ: integer := 0;
constant CSR_OP_BITS_WRITTEN: integer := 1;
constant CSR_OP_BITS_OPA: integer := 2;
constant CSR_OP_BITS_OPB: integer := 3;
constant CSR_OP_BITS_IMM: integer := 4;

constant CSR_MAINOP_WR: std_logic_vector(1 downto 0) := "01";
constant CSR_MAINOP_SET: std_logic_vector(1 downto 0) := "10";
constant CSR_MAINOP_CLEAR: std_logic_vector(1 downto 0) := "11";

constant CSR_OP_WR: std_logic_vector(4 downto 0) := "00100";
constant CSR_OP_W:  std_logic_vector(4 downto 0) := "00101";
constant CSR_OP_R:  std_logic_vector(4 downto 0) := "00110";

constant CSR_OP_SET_WR: std_logic_vector(4 downto 0) := "01000";
constant CSR_OP_SET_W:  std_logic_vector(4 downto 0) := "01001";
constant CSR_OP_SET_R:  std_logic_vector(4 downto 0) := "01010";

constant CSR_OP_CLEAR_WR: std_logic_vector(4 downto 0) := "01100";
constant CSR_OP_CLEAR_W:  std_logic_vector(4 downto 0) := "01101";
constant CSR_OP_CLEAR_R:  std_logic_vector(4 downto 0) := "01110";

constant CSR_OP_IMM_WR: std_logic_vector(4 downto 0) := "10100";
constant CSR_OP_IMM_W:  std_logic_vector(4 downto 0) := "10101";
constant CSR_OP_IMM_R:  std_logic_vector(4 downto 0) := "10110";

constant CSR_OP_IMM_SET_WR: std_logic_vector(4 downto 0) := "11000";
constant CSR_OP_IMM_SET_W:  std_logic_vector(4 downto 0) := "11001";
constant CSR_OP_IMM_SET_R:  std_logic_vector(4 downto 0) := "11010";

constant CSR_OP_IMM_CLEAR_WR: std_logic_vector(4 downto 0) := "11100";
constant CSR_OP_IMM_CLEAR_W:  std_logic_vector(4 downto 0) := "11101";
constant CSR_OP_IMM_CLEAR_R:  std_logic_vector(4 downto 0) := "11110";

end constants;

package body constants is
 
end constants;
